#ifndef MATH_UTIL_H
#define MATH_UTIL_H

#include <cstdint>
namespace Localization
{
namespace PointLoc
{
class MathUtil {
public:
  MathUtil() = delete;
  ~MathUtil() = delete;
  MathUtil(MathUtil const&) = delete;
  MathUtil& operator=(MathUtil const&) = delete;
  MathUtil(MathUtil&&) = delete;
  MathUtil& operator=(MathUtil&&) = delete;

  /// @brief Swap between big and little endian
  ///
  /// @param x  number
  ///
  /// @return swapped number
  static inline std::uint32_t SwapEndian(std::uint32_t x) noexcept {
    return (x >> 24) | ((x & 0xFF0000) >> 8) |
        ((x & 0xFF00) << 8) | (x << 24);
  }

  /// @brief Get 1/sqrt(x)
  /// Error < 1e-4
  ///
  /// @param x  number
  /// @throw InvalidArgumentException  when x is 0
  ///
  /// @return 1/sqrt(x)
  static float InvSqrt(float x) noexcept(false);

  /// @brief Get 1/sqrt(x) with higher precision
  /// Error < 1e-6
  ///
  /// @param x  number
  /// @throw InvalidArgumentException  when x is 0
  ///
  /// @return 1/sqrt(x)
  static float InvSqrtPrecise(float x) noexcept(false);

  /// @brief Get 1/sqrt(x)
  /// Error < 1e-9
  ///
  /// @param x  number
  /// @throw InvalidArgumentException  when x is 0
  ///
  /// @return 1/sqrt(x)
  static double InvSqrt(double x) noexcept(false);

  /// @brief Get 1/sqrt(x) with higher precision
  /// Error < 1e-12
  ///
  /// @param x  number
  /// @throw InvalidArgumentException  when x is 0
  ///
  /// @return 1/sqrt(x)
  static double InvSqrtPrecise(double x) noexcept(false);
};

} // namespace PointLoc
} // namespace Localization
#endif
